<?php

namespace Ulmus\User\Authorize;

use Psr\Http\Message\ServerRequestInterface;
use Ulmus\User\Common\AuthorizeContentTypeEnum;
use Ulmus\User\Entity\{ DigestAuthUserInterface, UserInterface };

class HeaderAuthentication implements AuthorizeMethodInterface
{
    public function connect(ServerRequestInterface $request, UserInterface $user): bool
    {
        if (null !== ( $auth = $request->getHeaderLine('Authorization') )) {
            list($method, $value) = explode(' ', $auth, 2) + [ null, null ];

            switch(strtolower($method)) {
                case "basic":
                    $methodObj = new Header\BasicMethod($user, $value);
                break;

                case "digest":
                    if (! $user instanceof DigestAuthUserInterface) {
                        throw new \RuntimeException("Your user entity must provide a valid hash of `user:realm:password` ");
                    }

                    $method = new Header\DigestMethod($user, $value);
                break;

                case "bearer":
                    $method = new Header\BearerMethod($user, $value);
                break;

                default:
                    throw new \InvalidArgumentException("An authentication method must be provided");
            }
        }

        return isset($methodObj) && $methodObj->execute($request);
    }


    public function catchRequest(ServerRequestInterface $request) : bool
    {
        foreach(array_merge($request->getHeader('Accept'), $request->getHeader('Content-Type')) as $accept) {
            foreach(explode(',', $accept) as $contentType) {
                if ( AuthorizeContentTypeEnum::tryFrom(strtolower($contentType)) ) {
                    return true;
                }
            }
        }

        return false;
    }
}