<?php

namespace Ulmus\User\Middleware;

use Psr\Http\{
    Message\ResponseInterface,
    Message\ServerRequestInterface,
    Server\MiddlewareInterface,
    Server\RequestHandlerInterface
};
use Ulmus\User\Entity\UserInterface;
use Ulmus\User\Authorize\PostRequestAuthentication;
use Ulmus\User\Lib\Authenticate;

class PostRequestAuthenticationMiddleware implements MiddlewareInterface
{
    protected PostRequestAuthentication $authenticator;

    public function __construct(
        protected UserInterface     $entity,
        protected \Closure          $loginFailedResponse,
        PostRequestAuthentication   $authenticator = null,
    ) {
        $this->authenticator = $authenticator ?: new PostRequestAuthentication(new Authenticate());
    }

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $this->authenticator->authenticate->rememberMe();

        if ( $this->authenticator->catchRequest($request) ) {
            if ( ! $this->authenticator->connect($request, $this->entity) ) {
                return call_user_func($this->loginFailedResponse, "Login failed");
            }
        }

        return $handler->handle($request);
    }
}
